@@ -215,7 +215,13 @@ bool uvm_va_space_mm_enabled(uvm_va_space_t *va_space)
 
     static struct mmu_notifier_ops uvm_mmu_notifier_ops_ats =
     {
+#if defined(NV_MMU_NOTIFIER_OPS_HAS_INVALIDATE_RANGE)
         .invalidate_range = uvm_mmu_notifier_invalidate_range_ats,
+#elif defined(NV_MMU_NOTIFIER_OPS_HAS_ARCH_INVALIDATE_SECONDARY_TLBS)
+        .arch_invalidate_secondary_tlbs = uvm_mmu_notifier_invalidate_range_ats,
+#else
+        #error One of invalidate_range/arch_invalid_secondary must be present
+#endif
     };
 
     static int uvm_mmu_notifier_register(uvm_va_space_mm_t *va_space_mm)
@@ -274,6 +280,28 @@ NV_STATUS uvm_va_space_mm_register(uvm_va_space_t *va_space)
         }
     }
 
+    if ((UVM_IS_CONFIG_HMM() || UVM_HMM_RANGE_FAULT_SUPPORTED()) &&
+        uvm_va_space_pageable_mem_access_supported(va_space)) {
+
+        #if UVM_CAN_USE_MMU_NOTIFIERS()
+            // Initialize MMU interval notifiers for this process. This allows
+            // mmu_interval_notifier_insert() to be called without holding the
+            // mmap_lock for write.
+            // Note: there is no __mmu_notifier_unregister(), this call just
+            // allocates memory which is attached to the mm_struct and freed
+            // when the mm_struct is freed.
+            ret = __mmu_notifier_register(NULL, current->mm);
+            if (ret) {
+                // Inform uvm_va_space_mm_unregister() that it has nothing to do.
+                uvm_mmdrop(va_space_mm->mm);
+                va_space_mm->mm = NULL;
+                return errno_to_nv_status(ret);
+            }
+        #else
+            UVM_ASSERT(0);
+        #endif
+    }
+
     return NV_OK;
 }
 
