@@ -126,16 +126,19 @@ void nvRmFreeEvoDma(NVDevEvoPtr pDevEvo, NVEvoDmaPtr pDma);
 NvBool nvRmQueryDpAuxLog(NVDispEvoRec *pDispEvo, NvS64 *pValue);
 NvU64 nvRmGetGpuTime(NVDevEvoPtr pDevEvo);
 NvBool nvRmSetGc6Allowed(NVDevEvoPtr pDevEvo, NvBool allowed);
-NvU32 nvRmAddRgLine1Callback(
-    const NVDispEvoRec *pDispEvo,
-    NvU32 head,
-    NV0092_REGISTER_RG_LINE_CALLBACK_FN pCallback);
+NVRgLine1CallbackPtr
+nvRmAddRgLine1Callback(NVDispEvoRec *pDispEvo,
+                       NvU32 head,
+                       NVRgLine1CallbackProc pCallbackProc,
+                       void *pUserData);
 void nvRmRemoveRgLine1Callback(const NVDispEvoRec *pDispEvo,
-                               NvU32 callbackObjectHandle);
+                               NVRgLine1CallbackPtr pCallback);
+
 NvU32 nvRmAddVBlankCallback(
     const NVDispEvoRec *pDispEvo,
     NvU32 head,
-    OSVBLANKCALLBACKPROC pCallback);
+    OSVBLANKCALLBACKPROC pCallback,
+    void *pParam2);
 void nvRmRemoveVBlankCallback(const NVDispEvoRec *pDispEvo,
                               NvU32 callbackObjectHandle);
 void nvRmMuxInit(NVDevEvoPtr pDevEvo);
@@ -147,6 +150,9 @@ NvMuxState nvRmMuxState(const NVDpyEvoRec *pDpyEvo);
 void nvRmRegisterBacklight(NVDispEvoRec *pDispEvo);
 void nvRmUnregisterBacklight(NVDispEvoRec *pDispEvo);
 
+void nvRmAllocCoreRGSyncpts(NVDevEvoPtr pDevEvo);
+void nvRmFreeCoreRGSyncpts(NVDevEvoPtr pDevEvo);
+
 #ifdef __cplusplus
 };
 #endif
