@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2000-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2000-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,6 +24,8 @@
 #ifndef NVPCIE_H
 #define NVPCIE_H
 
+
+
 /**************** Resource Manager Defines and Structures ******************\
 *                                                                           *
 *       Private PCI Express related defines and structures.                 *
@@ -53,6 +55,8 @@
 #define PCI_INVALID_SUBVENDORID 0xFFFF
 #define PCI_INVALID_SUBDEVICEID 0xFFFF
 
+#define PCI_IS_VENDORID_VALID(id)   (((id) != 0x0000) && ((id) != 0xFFFF))
+
 #define PCI_CLASS_BRIDGE_DEV    0x06
 #define PCI_SUBCLASS_BR_HOST    0x00
 #define PCI_MULTIFUNCTION       0x80
@@ -220,6 +224,20 @@
 
 #define NV_PCI_VIRTUAL_P2P_APPROVAL_SIGNATURE                   0x00503250
 
+#define NV_PCI_VIRTUAL_CONFIG_BITS_CAP_0                        0x000000D4
+#define NV_PCI_VIRTUAL_CONFIG_BITS_CAP_0_ID                            7:0
+#define NV_PCI_VIRTUAL_CONFIG_BITS_CAP_0_NEXT                         15:8
+#define NV_PCI_VIRTUAL_CONFIG_BITS_CAP_0_LENGTH                      23:16
+#define NV_PCI_VIRTUAL_CONFIG_BITS_CAP_0_SIG_LO                      31:24
+#define NV_PCI_VIRTUAL_CONFIG_BITS_CAP_1                        0x000000D8
+#define NV_PCI_VIRTUAL_CONFIG_BITS_CAP_1_SIG_HI                       15:0
+#define NV_PCI_VIRTUAL_CONFIG_BITS_CAP_1_VALUE                       31:16
+
+#define NV_PCI_VIRTUAL_CONFIG_BITS_SIGNATURE                    0x00535442  //"BTS"
+
+// Allocation of _VALUE bits
+#define NV_PCI_VIRTUAL_CONFIG_BITS_PCI_EGRESS_POISON_ENABLE            0:0
+
 // Chipset-specific definitions.
 // Intel SantaRosa definitions
 #define INTEL_2A00_CONFIG_SPACE_BASE        0x60
@@ -470,7 +488,8 @@ typedef struct
     CL_PCIE_DC_CAPABILITY_MAP_ENTRY entries[PCI_MAX_CAPS];
 } CL_PCIE_DC_CAPABILITY_MAP;
 
-struct OBJCL;
+typedef struct OBJCL OBJCL;
+typedef struct OBJGPU OBJGPU;
 // root port setup functions
 NV_STATUS Broadcom_HT2100_setupFunc(OBJGPU *, OBJCL*);
 
@@ -494,6 +513,6 @@ NV_STATUS AMD_RP1630_setupFunc(OBJGPU *, OBJCL*);
 NV_STATUS AMD_RP1483_setupFunc(OBJGPU *, OBJCL*);
 
 // Determines if the GPU is in a multi-GPU board based on devid checks
-NvBool gpuIsMultiGpuBoard(OBJGPU *, NvBool *);
+NvBool gpuIsMultiGpuBoard(OBJGPU *);
 
 #endif // NVPCIE_H
