@@ -59,6 +59,7 @@
 #include <gpu/gsp/kernel_gsp.h>
 #include "liblogdecode.h"
 #include <gpu/fsp/kern_fsp.h>
+#include  <gpu/gsp/kernel_gsp.h>
 
 #include <mem_mgr/virt_mem_mgr.h>
 #include <virtualization/kernel_vgpu_mgr.h>
@@ -80,6 +81,8 @@
 #include <class/cl2080.h>
 #include <class/cl402c.h>
 
+#include <gpu/conf_compute/conf_compute.h>
+
 #include <gpu/dce_client/dce_client.h>
 // RMCONFIG: need definition of REGISTER_ALL_HALS()
 #include "g_hal_register.h"
@@ -383,6 +386,13 @@ osHandleGpuLost
 
         gpuSetDisconnectedProperties(pGpu);
 
+        if (IS_GSP_CLIENT(pGpu))
+        {
+            // Notify all channels of the error so that UVM can fail gracefully
+            KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu);
+            kgspRcAndNotifyAllChannels(pGpu, pKernelGsp, ROBUST_CHANNEL_GPU_HAS_FALLEN_OFF_THE_BUS, NV_FALSE);
+        }
+
         // Trigger the OS's PCI recovery mechanism
         if (nv_pci_trigger_recovery(nv) != NV_OK)
         {
@@ -476,6 +486,8 @@ RmInitGpuInfoWithRmApi
              NV2080_CTRL_GPU_INFO_INDEX_DMABUF_CAPABILITY_YES);
     }
 
+    nv->coherent = pGpu->getProperty(pGpu, PDB_PROP_GPU_COHERENT_CPU_MAPPING);
+
     portMemFree(pGpuInfoParams);
 
     // UNLOCK: release GPUs lock
@@ -896,14 +908,12 @@ RmInitNvDevice(
         return;
     }
 
-    os_disable_console_access();
 
     status->rmStatus = gpumgrStateInitGpu(pGpu);
     if (status->rmStatus != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "*** Cannot initialize the device\n");
         RM_SET_ERROR(*status, RM_INIT_GPU_STATE_INIT_FAILED);
-        os_enable_console_access();
         return;
     }
     nvp->flags |= NV_INIT_FLAG_GPU_STATE;
@@ -933,13 +943,10 @@ RmInitNvDevice(
         NV_PRINTF(LEVEL_ERROR,
                   "*** Cannot load state into the device\n");
         RM_SET_ERROR(*status, RM_INIT_GPU_LOAD_FAILED);
-        os_enable_console_access();
         return;
     }
     nvp->flags |= NV_INIT_FLAG_GPU_STATE_LOAD;
 
-    os_enable_console_access();
-
     status->rmStatus = gpuPerformUniversalValidation_HAL(pGpu);
     if (status->rmStatus != NV_OK)
     {
@@ -1156,6 +1163,7 @@ NvBool RmInitPrivateState(
     nv_priv_t *nvp;
     NvU32 gpuId;
     NvU32 pmc_boot_0 = 0;
+    NvU32 pmc_boot_1 = 0;
     NvU32 pmc_boot_42 = 0;
 
     NV_SET_NV_PRIV(pNv, NULL);
@@ -1173,6 +1181,7 @@ NvBool RmInitPrivateState(
         }
 
         pmc_boot_0 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_0);
+        pmc_boot_1 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_1);
         pmc_boot_42 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_42);
 
         os_unmap_kernel_space(pNv->regs->map_u, os_page_size);
@@ -1212,6 +1221,7 @@ NvBool RmInitPrivateState(
     os_mem_set(nvp, 0, sizeof(*nvp));
     nvp->status = NV_ERR_INVALID_STATE;
     nvp->pmc_boot_0 = pmc_boot_0;
+    nvp->pmc_boot_1 = pmc_boot_1;
     nvp->pmc_boot_42 = pmc_boot_42;
     NV_SET_NV_PRIV(pNv, nvp);
 
@@ -1230,7 +1240,7 @@ void RmClearPrivateState(
     nv_i2c_adapter_entry_t i2c_adapters[MAX_I2C_ADAPTERS];
     nv_dynamic_power_t dynamicPowerCopy;
     NvU32 x = 0;
-    NvU32 pmc_boot_0, pmc_boot_42;
+    NvU32 pmc_boot_0, pmc_boot_1, pmc_boot_42;
 
     //
     // Do not clear private state after GPU resets, it is used while
@@ -1248,6 +1258,7 @@ void RmClearPrivateState(
     pRegistryCopy = nvp->pRegistry;
     dynamicPowerCopy = nvp->dynamic_power;
     pmc_boot_0 = nvp->pmc_boot_0;
+    pmc_boot_1 = nvp->pmc_boot_1;
     pmc_boot_42 = nvp->pmc_boot_42;
 
     for (x = 0; x < MAX_I2C_ADAPTERS; x++)
@@ -1263,6 +1274,7 @@ void RmClearPrivateState(
     nvp->pRegistry = pRegistryCopy;
     nvp->dynamic_power = dynamicPowerCopy;
     nvp->pmc_boot_0 = pmc_boot_0;
+    nvp->pmc_boot_1 = pmc_boot_1;
     nvp->pmc_boot_42 = pmc_boot_42;
 
     for (x = 0; x < MAX_I2C_ADAPTERS; x++)
@@ -1381,7 +1393,8 @@ static NvBool RmUnixAllocRmApi(
                     NV01_NULL_OBJECT,
                     NV01_NULL_OBJECT,
                     NV01_ROOT,
-                    &nv->rmapi.hClient) != NV_OK)
+                    &nv->rmapi.hClient,
+                    sizeof(nv->rmapi.hClient)) != NV_OK)
     {
         goto fail;
     }
@@ -1405,7 +1418,8 @@ static NvBool RmUnixAllocRmApi(
                     nv->rmapi.hClient,
                     &nv->rmapi.hDevice,
                     NV01_DEVICE_0,
-                    &deviceParams) != NV_OK)
+                    &deviceParams,
+                    sizeof(deviceParams)) != NV_OK)
     {
         goto fail;
     }
@@ -1418,7 +1432,8 @@ static NvBool RmUnixAllocRmApi(
                     nv->rmapi.hDevice,
                     &nv->rmapi.hSubDevice,
                     NV20_SUBDEVICE_0,
-                    &subDeviceParams) != NV_OK)
+                    &subDeviceParams,
+                    sizeof(subDeviceParams)) != NV_OK)
     {
         goto fail;
     }
@@ -1433,7 +1448,8 @@ static NvBool RmUnixAllocRmApi(
                     nv->rmapi.hSubDevice,
                     &nv->rmapi.hI2C,
                     NV40_I2C,
-                    NULL) != NV_OK)
+                    NULL,
+                    0) != NV_OK)
     {
         nv->rmapi.hI2C = 0;
     }
@@ -1449,7 +1465,8 @@ static NvBool RmUnixAllocRmApi(
                     nv->rmapi.hDevice,
                     &nv->rmapi.hDisp,
                     NV04_DISPLAY_COMMON,
-                    NULL) != NV_OK)
+                    NULL,
+                    0) != NV_OK)
     {
         nv->rmapi.hDisp = 0;
     }
@@ -1461,6 +1478,59 @@ fail:
     return NV_FALSE;
 }
 
+static NV_STATUS RmFetchGspRmImages
+(
+    nv_state_t    *nv,
+    GSP_FIRMWARE  *pGspFw,
+    const void   **gspFwHandle,
+    const void   **gspFwLogHandle
+)
+{
+    nv_firmware_chip_family_t chipFamily;
+    nv_priv_t *nvp = NV_GET_NV_PRIV(nv);
+    NvU32 gpuArch = (DRF_VAL(_PMC, _BOOT_42, _ARCHITECTURE, nvp->pmc_boot_42) <<
+                     GPU_ARCH_SHIFT);
+    NvU32 gpuImpl = DRF_VAL(_PMC, _BOOT_42, _IMPLEMENTATION, nvp->pmc_boot_42);
+
+    chipFamily = nv_firmware_get_chip_family(gpuArch, gpuImpl);
+
+    portMemSet(pGspFw, 0, sizeof(*pGspFw));
+
+    *gspFwHandle = nv_get_firmware(nv, NV_FIRMWARE_TYPE_GSP,
+                                   chipFamily,
+                                   &pGspFw->pBuf,
+                                   &pGspFw->size);
+    if (*gspFwHandle == NULL &&
+        !nv->allow_fallback_to_monolithic_rm)
+    {
+        NV_PRINTF(LEVEL_ERROR, "No firmware image found\n");
+        return NV_ERR_NOT_SUPPORTED;
+    }
+    else if (*gspFwHandle != NULL)
+    {
+#if LIBOS_LOG_DECODE_ENABLE
+        if (nv->enable_firmware_logs)
+        {
+            *gspFwLogHandle = nv_get_firmware(nv, NV_FIRMWARE_TYPE_GSP_LOG,
+                                              chipFamily,
+                                              &pGspFw->pLogElf,
+                                              &pGspFw->logElfSize);
+            if (*gspFwLogHandle == NULL)
+            {
+                NV_PRINTF(LEVEL_ERROR, "Failed to load gsp_log_*.bin, no GSP-RM logs will be printed (non-fatal)\n");
+            }
+        }
+#endif
+        nv->request_fw_client_rm = NV_TRUE;
+    }
+    else
+    {
+        nv->request_fw_client_rm = NV_FALSE;
+    }
+
+    return NV_OK;
+}
+
 NvBool RmInitAdapter(
     nv_state_t *nv
 )
@@ -1475,6 +1545,7 @@ NvBool RmInitAdapter(
     KernelDisplay  *pKernelDisplay;
     const void     *gspFwHandle = NULL;
     const void     *gspFwLogHandle = NULL;
+    NvBool          consoleDisabled = NV_FALSE;
 
     GSP_FIRMWARE    gspFw = {0};
     PORT_UNREFERENCED_VARIABLE(gspFw);
@@ -1524,46 +1595,14 @@ NvBool RmInitAdapter(
     //
     if (nv->request_firmware)
     {
-        NvU32 gpuArch = (DRF_VAL(_PMC, _BOOT_42, _ARCHITECTURE, nvp->pmc_boot_42) <<
-                         GPU_ARCH_SHIFT);
-        NvU32 gpuImpl = DRF_VAL(_PMC, _BOOT_42, _IMPLEMENTATION, nvp->pmc_boot_42);
-
-        nv_firmware_chip_family_t chipFamily = nv_firmware_get_chip_family(gpuArch, gpuImpl);
-
         nv_set_dma_address_size(nv, NV_GSP_GPU_MIN_SUPPORTED_DMA_ADDR_WIDTH);
 
-        gspFwHandle = nv_get_firmware(nv, NV_FIRMWARE_TYPE_GSP,
-                                      chipFamily,
-                                      &gspFw.pBuf,
-                                      &gspFw.size);
-        if (gspFwHandle == NULL &&
-            !nv->allow_fallback_to_monolithic_rm)
+        status.rmStatus = RmFetchGspRmImages(nv, &gspFw, &gspFwHandle, &gspFwLogHandle);
+        if (status.rmStatus != NV_OK)
         {
             RM_SET_ERROR(status, RM_INIT_FIRMWARE_FETCH_FAILED);
             goto shutdown;
         }
-        else if (gspFwHandle != NULL)
-        {
-#if LIBOS_LOG_DECODE_ENABLE
-            if (nv->enable_firmware_logs)
-            {
-                gspFwLogHandle = nv_get_firmware(nv, NV_FIRMWARE_TYPE_GSP_LOG,
-                                                 chipFamily,
-                                                 &gspFw.pLogElf,
-                                                 &gspFw.logElfSize);
-                if (gspFwLogHandle == NULL)
-                {
-                    NV_PRINTF(LEVEL_ERROR, "Failed to load gsp_log_*.bin, no GSP-RM logs will be printed (non-fatal)\n");
-                }
-            }
-#endif
-
-            nv->request_fw_client_rm = NV_TRUE;
-        }
-        else
-        {
-            nv->request_fw_client_rm = NV_FALSE;
-        }
     }
 
     // initialize the RM device register mapping
@@ -1606,6 +1645,7 @@ NvBool RmInitAdapter(
         if (status.rmStatus != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR, "FSP boot command failed.\n");
+            RM_SET_ERROR(status, RM_INIT_FIRMWARE_INIT_FAILED);
             goto shutdown;
         }
     }
@@ -1614,6 +1654,16 @@ NvBool RmInitAdapter(
 
     RmInitAcpiMethods(pOS, pSys, pGpu);
 
+    //
+    // For GPU driving console, disable console access here, to ensure no console
+    // writes through BAR1 can interfere with physical RM's setup of BAR1
+    //
+    if (rm_get_uefi_console_status(nv))
+    {
+        os_disable_console_access();
+        consoleDisabled = NV_TRUE;
+    }
+
     //
     // If GSP fw RM support is enabled then start the GSP microcode
     // (including the task running the full instance of the RM) and
@@ -1661,6 +1711,17 @@ NvBool RmInitAdapter(
         initNbsiTable(pGpu);
     }
 
+    //
+    // Load GSP proxy if early init is required. We need to do this
+    // before we trigger a full gpuStateInit and gpuStateLoad in
+    // RmInitNvDevice
+    // TODO: Check bug 200744430
+    //
+    if (gpuIsCCFeatureEnabled(pGpu))
+    {
+        confComputeEarlyInit(pGpu, GPU_GET_CONF_COMPUTE(pGpu));
+    }
+
     // finally, initialize the device
     RmInitNvDevice(devicereference, &status);
     if (! RM_INIT_SUCCESS(status.initStatus) )
@@ -1678,6 +1739,12 @@ NvBool RmInitAdapter(
         goto shutdown;
     }
 
+    if (consoleDisabled)
+    {
+        os_enable_console_access();
+        consoleDisabled = NV_FALSE;
+    }
+
     // LOCK: acquire GPUs lock
     status.rmStatus = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE,
                                         RM_LOCK_MODULES_INIT);
@@ -1710,6 +1777,14 @@ NvBool RmInitAdapter(
         intrSetIntrEn(pIntr, INTERRUPT_TYPE_HARDWARE);
     }
 
+    // LOCK: acquire GPUs lock
+    status.rmStatus = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE,
+                                        RM_LOCK_MODULES_INIT);
+    if (status.rmStatus != NV_OK)
+    {
+        goto shutdown;
+    }
+
     KernelRc *pKernelRc = GPU_GET_KERNEL_RC(pGpu);
     // initialize the watchdog (disabled by default)
     status.rmStatus = pKernelRc != NULL ? krcWatchdogInit_HAL(pGpu, pKernelRc) :
@@ -1729,8 +1804,13 @@ NvBool RmInitAdapter(
         RM_SET_ERROR(status, RM_INIT_WATCHDOG_FAILED);
         NV_PRINTF(LEVEL_ERROR,
                   "krcWatchdogInit failed, bailing out of RmInitAdapter\n");
+        // UNLOCK: release GPUs lock
+        rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
         goto shutdown;
     }
+
+    // UNLOCK: release GPUs lock
+    rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
     nv_start_rc_timer(nv);
 
     nvp->status = NV_OK;
@@ -1860,6 +1940,11 @@ NvBool RmInitAdapter(
  shutdown:
     nv->flags &= ~NV_FLAG_IN_RECOVERY;
 
+    if (consoleDisabled)
+    {
+        os_enable_console_access();
+    }
+
     // call ShutdownAdapter to undo anything we've done above
     RmShutdownAdapter(nv);
 
