@@ -36,6 +36,8 @@
 
 #include "ctrl/ctrl2080/ctrl2080fb.h" // NV2080_CTRL_FB_INFO
 
+#include <stddef.h>
+
 typedef NV_STATUS RmAllocMemoryFunc(
     DEPRECATED_CONTEXT *pContext,
     NvHandle            hClient,
@@ -165,7 +167,7 @@ _rmAllocMemorySystem
 
     *pAddress = NvP64_NULL;
 
-    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, &allocParams);
+    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, &allocParams, sizeof(allocParams));
 
     if (status != NV_OK)
         return status;
@@ -214,7 +216,7 @@ _rmAllocMemorySystemDynamic
 
     // Try with NV01_MEMORY_SYSTEM_DYNAMIC for NV01_DEVICE_0 parents
     status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory,
-                               NV01_MEMORY_SYSTEM_DYNAMIC, &allocParams);
+                               NV01_MEMORY_SYSTEM_DYNAMIC, &allocParams, sizeof(allocParams));
 
     *pLimit = allocParams.limit;
 
@@ -255,7 +257,7 @@ _rmAllocMemorySystemOsDescriptor
         return NV_ERR_INVALID_FLAGS;
     }
 
-    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, &allocParams);
+    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, &allocParams, sizeof(allocParams));
 
     return status;
 }
@@ -324,7 +326,7 @@ _rmAllocMemoryLocalUser
     //
 
     // First attempt: try to allocate NV01_MEMORY_LOCAL_PHYSICAL
-    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, NV01_MEMORY_LOCAL_PHYSICAL, &allocParams);
+    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, NV01_MEMORY_LOCAL_PHYSICAL, &allocParams, sizeof(allocParams));
 
     if (status == NV_OK)
     {
@@ -357,7 +359,7 @@ _rmAllocMemoryLocalUser
         // user-mode clients previously received hMemory of entire FB)
         //
         status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory,
-                                   NV01_MEMORY_SYSTEM_DYNAMIC, &virtAllocParams);
+                                   NV01_MEMORY_SYSTEM_DYNAMIC, &virtAllocParams, sizeof(virtAllocParams));
     }
 
     return status;
@@ -381,7 +383,7 @@ _rmAllocMemoryLocalPrivileged
 
     *pLimit = 0xFFFFFFFF; // not used by clients
 
-    return pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, 0);
+    return pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, NULL, 0);
 }
 
 static NV_STATUS
@@ -456,7 +458,7 @@ _rmAllocMemoryList
     COPY_FIELD(size);
     COPY_FIELD(align);
 
-    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, &allocParams);
+    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, &allocParams, sizeof(allocParams));
 
 done:
     if (pPageArray)
@@ -513,7 +515,7 @@ _rmAllocMemoryFromFlaObject
     COPY_FLA_FIELD(hExportHandle);
     COPY_FLA_FIELD(hExportClient);
 
-    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, &allocParams);
+    status = pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, &allocParams, sizeof(allocParams));
 
 done:
     if (pMemoryFla)
@@ -541,5 +543,5 @@ _rmAllocMemoryFramebufferConsole
     NvU64              *pLimit
 )
 {
-    return pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, 0);
+    return pContext->RmAlloc(pContext, hClient, hParent, &hMemory, hClass, NULL, 0);
 }
