@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2008-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2008-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -58,6 +58,8 @@
 #include "objtmr.h"
 #include "lib/base_utils.h"
 
+#include "gpu/conf_compute/conf_compute.h"
+
 #define SDK_ALL_CLASSES_INCLUDE_FULL_HEADER
 #include "g_allclasses.h"
 #undef SDK_ALL_CLASSES_INCLUDE_FULL_HEADER
@@ -152,6 +154,7 @@ static NV_STATUS _issueRpcAndWait(OBJGPU *pGpu, OBJRPC *pRpc)
 
     // should not be called in broadcast mode
     NV_ASSERT_OR_RETURN(!gpumgrGetBcEnabledStatus(pGpu), NV_ERR_INVALID_STATE);
+    NV_CHECK(LEVEL_ERROR, rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
 
     if (bProfileRPC)
     {
@@ -180,6 +183,9 @@ static NV_STATUS _issueRpcAndWait(OBJGPU *pGpu, OBJRPC *pRpc)
         osGetPerformanceCounter(&pNewEntry->rpcData.startTimeInNs);
     }
 
+    // For HCC, cache expectedFunc value before encrypting.
+    NvU32 expectedFunc = vgpu_rpc_message_header_v->function;
+
     status = rpcSendMessage(pGpu, pRpc);
     if (status != NV_OK)
     {
@@ -195,7 +201,8 @@ static NV_STATUS _issueRpcAndWait(OBJGPU *pGpu, OBJRPC *pRpc)
         return (status == NV_ERR_BUSY_RETRY) ? NV_ERR_GENERIC : status;
     }
 
-    status = rpcRecvPoll(pGpu, pRpc, vgpu_rpc_message_header_v->function);
+    // Use cached expectedFunc here because vgpu_rpc_message_header_v is encrypted for HCC.
+    status = rpcRecvPoll(pGpu, pRpc, expectedFunc);
     if (status != NV_OK)
     {
         if (status == NV_ERR_TIMEOUT)
@@ -310,6 +317,13 @@ static NV_STATUS _issueRpcLarge
         if (entryLength > remainingSize)
             entryLength = remainingSize;
 
+        ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
+        if (pCC != NULL && pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_ENCRYPT_ENABLED))
+        {
+            // Zero out the entire RPC message header to clear the state of previous chunk.
+            portMemSet(vgpu_rpc_message_header_v, 0, sizeof(rpc_message_header_v));
+        }
+
         portMemCopy(rpc_message, entryLength, pBuf8, entryLength);
 
         // Set the correct length for this queue entry.
@@ -1323,6 +1337,7 @@ NV_STATUS rpcGspSetSystemInfo_v17_00
         rpcInfo->upstreamAddressValid     = pGpu->gpuClData.upstreamPort.addr.valid;
 
         rpcInfo->hypervisorType           = hypervisorGetHypervisorType(pHypervisor);
+        rpcInfo->virtualConfigBits        = pGpu->virtualConfigBits;
         rpcInfo->bIsPassthru              = pGpu->bIsPassthru;
 
         // Fill in VF related GPU flags
@@ -1338,6 +1353,10 @@ NV_STATUS rpcGspSetSystemInfo_v17_00
         OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
         rpcInfo->sysTimerOffsetNs = pTmr->sysTimerOffsetNs;
 
+
+        // Indicate whether the driver supports NV2080_NOTIFIERS_UCODE_RESET event.
+        rpcInfo->bTdrEventSupported = pGpu->getProperty(pGpu, PDB_PROP_GPU_SUPPORTS_TDR_EVENT);
+
         status = _issueRpcAsync(pGpu, pRpc);
     }
 
@@ -1439,6 +1458,8 @@ NV_STATUS rpcDumpProtobufComponent_v18_12
     if (IS_GSP_CLIENT(pGpu))
     {
         rpc_dump_protobuf_component_v18_12 *rpc_params = &rpc_message->dump_protobuf_component_v18_12;
+        const NvU32 fixed_param_size = sizeof(rpc_message_header_v) + sizeof(*rpc_params);
+        NV_ASSERT_OR_RETURN(fixed_param_size <= pRpc->maxRpcSize, NV_ERR_INVALID_STATE);
 
         status = rpcWriteCommonHeader(pGpu, pRpc, NV_VGPU_MSG_FUNCTION_DUMP_PROTOBUF_COMPONENT,
                                     sizeof(*rpc_params));
@@ -1450,7 +1471,7 @@ NV_STATUS rpcDumpProtobufComponent_v18_12
         rpc_params->countOnly    = ((pPrbEnc->flags & PRB_COUNT_ONLY) != 0);
         rpc_params->bugCheckCode = pNvDumpState->bugCheckCode;
         rpc_params->internalCode = pNvDumpState->internalCode;
-        rpc_params->bufferSize   = NV_MIN(pRpc->maxRpcSize, prbEncBufLeft(pPrbEnc));
+        rpc_params->bufferSize   = NV_MIN(pRpc->maxRpcSize - fixed_param_size, prbEncBufLeft(pPrbEnc));
 
         status = _issueRpcAndWait(pGpu, pRpc);
 
@@ -1573,7 +1594,7 @@ NV_STATUS rpcRmApiControl_GSP
                 GPU_LOCK_FLAGS_SAFE_LOCK_UPGRADE, RM_LOCK_MODULES_RPC, &gpuMaskRelease));
     }
 
-    rmapiutilGetControlInfo(cmd, &ctrlFlags, &ctrlAccessRight);
+    rmapiutilGetControlInfo(cmd, &ctrlFlags, &ctrlAccessRight, NULL);
     bCacheable = rmapiControlIsCacheable(ctrlFlags, ctrlAccessRight, NV_TRUE);
 
     pCallContext = resservGetTlsCallContext();
@@ -1990,3 +2011,27 @@ done:
     }
     return status;
 }
+
+/*
+ * Sends ack from CPU-RM to GSP-RM that ECC error
+ * notifier write has completed.
+ */
+NV_STATUS rpcEccNotifierWriteAck_v23_05
+(
+    OBJGPU                *pGpu,
+    OBJRPC                *pRpc
+)
+{
+    NV_STATUS status = NV_ERR_NOT_SUPPORTED;
+
+    if (IS_GSP_CLIENT(pGpu))
+    {
+        status = rpcWriteCommonHeader(pGpu, pRpc, NV_VGPU_MSG_FUNCTION_ECC_NOTIFIER_WRITE_ACK, 0);
+        if (status != NV_OK)
+            return status;
+
+        status = _issueRpcAsync(pGpu, pRpc);
+    }
+
+    return status;
+}
