@@ -39,6 +39,7 @@
 #include <gpu/gpu.h>
 #include "kernel/gpu/intr/intr.h"
 #include <gpu/bif/kernel_bif.h>
+#include "gpu/disp/kern_disp.h"
 
 //
 // GPU lock
@@ -563,12 +564,11 @@ _rmGpuLocksAcquire(NvU32 gpuMask, NvU32 flags, NvU32 module, void *ra, NvU32 *pG
         // If we own a higher order lock than one of the needed ones, we are
         // violating the locking order and need to do a conditional acquire
         // clz32(0) == ctz(0) == 32:
-        //    owned=0b00110000, needed=0b00001100: (4  < (32-28)), bCond=FALSE
-        //    owned=0b00110010, needed=0b00001100: (1  < (32-28)), bCond=TRUE
-        //    owned=0b00010000, needed=0b11000011: (4  < (32-24)), bCond=TRUE
-        //    owned=0b00000000, needed=0b00001100: (32 < (32-28)), bCond=FALSE
-        //    owned=0b00000001, needed=0b00000000: (0  < (32-32)), bCond=FALSE
-        if (portUtilCountTrailingZeros32(ownedMask) < (32-portUtilCountLeadingZeros32(gpuMask)))
+        //    owned=0b00001100, needed=0b00110000: ((32-28) >  4), bCond=FALSE
+        //    owned=0b00001100, needed=0b00110010: ((32-28) >  1), bCond=TRUE
+        //    owned=0b11000011, needed=0b00010000: ((32-24) >  4), bCond=TRUE
+        //    owned=0b00000000, needed=0b00000001: ((32-32) >  0), bCond=FALSE
+        if ((32-portUtilCountLeadingZeros32(ownedMask)) > portUtilCountTrailingZeros32(gpuMask))
         {
             bCondAcquireCheck = NV_TRUE;
         }
@@ -1487,6 +1487,9 @@ rmGpuGroupLockRelease(GPU_MASK gpuMask, NvU32 flags)
     OBJSYS *pSys = SYS_GET_INSTANCE();
     OBJGPU *pDpcGpu = NULL;
 
+    if (gpuMask == 0)
+        return NV_SEMA_RELEASE_SUCCEED;
+
     //
     // QuadroSync (previously known as GSync) is a cross GPU feature that
     // synchronizes display across multiple GPUs.  See changelist 16809243.  If
