@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -52,7 +52,6 @@
 #include "deprecated/rmapi_deprecated.h"
 #include "nvRmReg.h"
 
-
 //
 // Watchdog object ids
 //
@@ -107,7 +106,6 @@
 
 #define SUBDEVICE_MASK_ALL DRF_MASK(NV906F_DMA_SET_SUBDEVICE_MASK_VALUE)
 
-
 NV_STATUS
 krcWatchdogChangeState_IMPL
 (
@@ -402,7 +400,7 @@ krcWatchdogShutdown_IMPL
 
     //
     // Make sure to clear any old watchdog data this also clears
-    // WATCHDOG_FLAGS_INITIALIZED
+    // WATCHDOG_FLAGS_INITIALIZED, bHandleValid, and hClient
     //
     portMemSet(&pKernelRc->watchdog, 0, sizeof pKernelRc->watchdog);
     portMemSet(&pKernelRc->watchdogChannelInfo, 0,
@@ -445,8 +443,8 @@ krcWatchdogInit_IMPL
     NvU32           pushBufBytes;
     NvU32           allocationSize;
     NvU32           ctrlSize;
-    NvU32           flags;
     NV_STATUS       status;
+    RsClient       *pClient;
     KernelChannel  *pKernelChannel;
     NvBool          bCacheSnoop;
     RM_API         *pRmApi = rmGpuLockIsOwner() ?
@@ -490,6 +488,14 @@ krcWatchdogInit_IMPL
     portMemSet(&pKernelRc->watchdogChannelInfo, 0,
                sizeof pKernelRc->watchdogChannelInfo);
 
+    // Bug 4088184 WAR: release GPU lock before allocating NV01_ROOT
+    if (rmGpuLockIsOwner())
+    {
+        bAcquireLock = NV_TRUE;
+        rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
+        pRmApi = rmapiGetInterface(RMAPI_API_LOCK_INTERNAL);
+    }
+
     // Allocate a root.
     {
         hClient = NV01_NULL_OBJECT;
@@ -498,7 +504,8 @@ krcWatchdogInit_IMPL
                                     NV01_NULL_OBJECT /* hParent */,
                                     NV01_NULL_OBJECT /* hObject */,
                                     NV01_ROOT,
-                                    &hClient) != NV_OK)
+                                    &hClient,
+                                    sizeof(hClient)) != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING, "Unable to allocate a watchdog client\n");
             return NV_ERR_GENERIC;
@@ -510,6 +517,21 @@ krcWatchdogInit_IMPL
             status = NV_ERR_NO_MEMORY;
             goto error;
         }
+        pKernelRc->watchdog.hClient = hClient;
+        pKernelRc->watchdog.bHandleValid = NV_TRUE;
+    }
+
+    if (bAcquireLock)
+    {
+        status = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_RC);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "failed to grab RM-Lock\n");
+            DBG_BREAKPOINT();
+            goto error;
+        }
+        pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+        bAcquireLock = NV_FALSE;
     }
 
     // Alloc device
@@ -525,7 +547,8 @@ krcWatchdogInit_IMPL
                                          hClient            /* hParent */,
                                          WATCHDOG_DEVICE_ID /* hObject */,
                                          NV01_DEVICE_0,
-                                         pNv0080);
+                                         pNv0080,
+                                         sizeof(*pNv0080));
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING, "Unable to allocate a watchdog device\n");
@@ -548,7 +571,8 @@ krcWatchdogInit_IMPL
             WATCHDOG_DEVICE_ID                             /* hParent */,
             (WATCHDOG_SUB_DEVICE_0_ID + subDeviceInstance) /* hObject */,
             NV20_SUBDEVICE_0,
-            pNv2080);
+            pNv2080,
+            sizeof(*pNv2080));
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING,
@@ -627,7 +651,8 @@ krcWatchdogInit_IMPL
                                          WATCHDOG_DEVICE_ID      /* hParent */,
                                          WATCHDOG_VIRTUAL_CTX_ID /* hObject */,
                                          NV01_MEMORY_VIRTUAL,
-                                         pVirtual);
+                                         pVirtual,
+                                         sizeof(*pVirtual));
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING,
@@ -655,45 +680,108 @@ krcWatchdogInit_IMPL
     bCacheSnoop = FLD_TEST_REF(BIF_DMA_CAPS_SNOOP, _CTXDMA,
                                kbifGetDmaCaps(pGpu, pKernelBif));
 
-    // Allocate memory for the notifiers and pushbuffer.
-    flags = ((bCacheSnoop ? DRF_DEF(OS02, _FLAGS, _COHERENCY, _CACHED) :
-                            DRF_DEF(OS02, _FLAGS, _COHERENCY, _UNCACHED)) |
-             DRF_DEF(OS02, _FLAGS, _LOCATION, _PCI) |
-             DRF_DEF(OS02, _FLAGS, _PHYSICALITY, _NONCONTIGUOUS));
-    if ((pKernelRc->watchdog.flags & WATCHDOG_FLAGS_ALLOC_UNCACHED_PCI) != 0)
-    {
-        flags = FLD_SET_DRF(OS02, _FLAGS, _COHERENCY, _UNCACHED, flags);
-    }
-
     {
         NV_MEMORY_ALLOCATION_PARAMS *pMem = &pParams->mem;
+        NvU32 hClass = NV01_MEMORY_SYSTEM;
 
         portMemSet(pMem, 0, sizeof *pMem);
         pMem->owner = HEAP_OWNER_RM_CLIENT_GENERIC;
         pMem->size  = allocationSize;
         pMem->type  = NVOS32_TYPE_IMAGE;
 
-        // TODO - migrate to use newer OS32 flags inline JIRA CORERM-4212
-        status = RmDeprecatedConvertOs02ToOs32Flags(flags,
-                                                    &pMem->attr,
-                                                    &pMem->attr2,
-                                                    &pMem->flags);
-        if (status != NV_OK)
+        pMem->attr2 = DRF_DEF(OS32, _ATTR2, _GPU_CACHEABLE, _NO);
+
+        // Apply registry overrides to channel pushbuffer.
+        switch (DRF_VAL(_REG_STR_RM, _INST_LOC_4, _CHANNEL_PUSHBUFFER, pGpu->instLocOverrides4))
         {
-            NV_PRINTF(LEVEL_ERROR, "Invalid flags\n");
-            goto error;
+            case NV_REG_STR_RM_INST_LOC_4_CHANNEL_PUSHBUFFER_VID:
+                hClass = NV01_MEMORY_LOCAL_USER;
+                pMem->attr |= DRF_DEF(OS32, _ATTR, _LOCATION,  _VIDMEM) |
+                              DRF_DEF(OS32, _ATTR, _COHERENCY, _UNCACHED);
+                break;
+
+            case NV_REG_STR_RM_INST_LOC_4_CHANNEL_PUSHBUFFER_COH:
+                hClass = NV01_MEMORY_SYSTEM;
+                pMem->attr |= DRF_DEF(OS32, _ATTR, _LOCATION,  _PCI)    |
+                              DRF_DEF(OS32, _ATTR, _COHERENCY, _CACHED) |
+                              DRF_DEF(OS32, _ATTR, _PHYSICALITY, _NONCONTIGUOUS);
+                break;
+
+            case NV_REG_STR_RM_INST_LOC_4_CHANNEL_PUSHBUFFER_NCOH:
+                hClass = NV01_MEMORY_SYSTEM;
+                pMem->attr |= DRF_DEF(OS32, _ATTR, _LOCATION,  _PCI)      |
+                              DRF_DEF(OS32, _ATTR, _COHERENCY, _UNCACHED) |
+                              DRF_DEF(OS32, _ATTR, _PHYSICALITY, _NONCONTIGUOUS);
+                break;
+
+            case NV_REG_STR_RM_INST_LOC_4_CHANNEL_PUSHBUFFER_DEFAULT:
+            default:
+                hClass = NV01_MEMORY_SYSTEM;
+                pMem->attr |= DRF_DEF(OS32, _ATTR, _LOCATION,  _PCI)      |
+                              DRF_DEF(OS32, _ATTR, _COHERENCY, _UNCACHED) |
+                              DRF_DEF(OS32, _ATTR, _PHYSICALITY, _NONCONTIGUOUS);
         }
 
+        if (bCacheSnoop && (hClass == NV01_MEMORY_SYSTEM))
+        {
+            pMem->attr = FLD_SET_DRF(OS32, _ATTR, _COHERENCY, _CACHED,
+                                     pMem->attr);
+        }
+
+        if (((pKernelRc->watchdog.flags & WATCHDOG_FLAGS_ALLOC_UNCACHED_PCI) != 0) &&
+            (hClass == NV01_MEMORY_SYSTEM))
+        {
+            pMem->attr = FLD_SET_DRF(OS32, _ATTR, _COHERENCY, _UNCACHED,
+                                     pMem->attr);
+        }
+
+        //
+        // When Hopper CC is enabled all RM internal sysmem allocations that are
+        // required to be accessed from GPU should be in unprotected memory
+        // All video allocations must be in CPR
+        //
+
+        //
+        // Allocate memory using vidHeapControl
+        //
+        // vidHeapControl calls should happen outside GPU locks. This is a PMA
+        // requirement as memory allocation calls may invoke eviction which UVM
+        // could get stuck behind GPU lock
+        //
+        if (hClass == NV01_MEMORY_LOCAL_USER && rmGpuLockIsOwner())
+        {
+            bAcquireLock = NV_TRUE;
+            rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
+            pRmApi = rmapiGetInterface(RMAPI_API_LOCK_INTERNAL);
+        }
+
+        // Allocate memory for the notifiers and pushbuffer
         status = pRmApi->AllocWithHandle(pRmApi,
                                          hClient            /* hClient */,
                                          WATCHDOG_DEVICE_ID /* hParent */,
                                          WATCHDOG_MEM_ID    /* hObject */,
-                                         NV01_MEMORY_SYSTEM,
-                                         pMem);
+                                         hClass,
+                                         pMem,
+                                         sizeof(*pMem));
+
+        if (bAcquireLock)
+        {
+            // Re-acquire the GPU locks
+            if (rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_RC) != NV_OK)
+            {
+                NV_PRINTF(LEVEL_ERROR, "failed to grab RM-Lock\n");
+                DBG_BREAKPOINT();
+                goto error;
+            }
+            pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+            bAcquireLock = NV_FALSE;
+        }
+
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING,
-                      "Unable to allocate system memory for watchdog\n");
+                      "Unable to allocate %s memory for watchdog\n",
+                      (hClass == NV01_MEMORY_LOCAL_USER) ? "video" : "system");
             goto error;
         }
 
@@ -708,7 +796,7 @@ krcWatchdogInit_IMPL
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING,
-                      "Unable to map system memory for watchdog\n");
+                      "Unable to map memory for watchdog\n");
             goto error;
         }
 
@@ -729,7 +817,7 @@ krcWatchdogInit_IMPL
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR,
-                      "Unable to map system memory into watchdog's heap\n");
+                      "Unable to map memory into watchdog's heap\n");
             goto error;
         }
     }
@@ -758,7 +846,8 @@ krcWatchdogInit_IMPL
                                          WATCHDOG_DEVICE_ID    /* hParent */ ,
                                          WATCHDOG_ERROR_DMA_ID /* hObject */,
                                          NV01_CONTEXT_DMA,
-                                         pCtxDma);
+                                         pCtxDma,
+                                         sizeof(*pCtxDma));
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING,
@@ -792,7 +881,8 @@ krcWatchdogInit_IMPL
                                          WATCHDOG_DEVICE_ID       /* hParent */,
                                          WATCHDOG_NOTIFIER_DMA_ID /* hObject */,
                                          NV01_CONTEXT_DMA,
-                                         pCtxDma);
+                                         pCtxDma,
+                                         sizeof(*pCtxDma));
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING, "Unable to set up watchdog's notifier\n");
@@ -825,6 +915,14 @@ krcWatchdogInit_IMPL
                 break;
         }
 
+        //
+        // When APM is enabled all RM internal allocations must to go to
+        // unprotected memory irrespective of vidmem or sysmem
+        // When Hopper CC is enabled all RM internal sysmem allocations that
+        // are required to be accessed from GPU should be in unprotected memory
+        // and all vidmem allocations must go to protected memory
+        //
+
         //
         // Allocate memory using vidHeapControl
         //
@@ -848,7 +946,8 @@ krcWatchdogInit_IMPL
             WATCHDOG_DEVICE_ID         /* hParent */,
             WATCHDOG_USERD_PHYS_MEM_ID /* hObject */,
             userdMemClass,
-            pMem);
+            pMem,
+            sizeof(*pMem));
 
         if (status != NV_OK)
         {
@@ -894,7 +993,8 @@ krcWatchdogInit_IMPL
             WATCHDOG_DEVICE_ID             /* hParent */,
             WATCHDOG_PUSHBUFFER_CHANNEL_ID /* hObject */,
             gpfifoObj,
-            pChannelGPFifo);
+            pChannelGPFifo,
+            sizeof(*pChannelGPFifo));
 
         if (bAcquireLock)
         {
@@ -970,18 +1070,27 @@ krcWatchdogInit_IMPL
         WATCHDOG_PUSHBUFFER_CHANNEL_ID /* hParent */,
         WATCHDOG_GROBJ_ID              /* hObject */,
         grObj,
-        NULL);
+        NULL,
+        0);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_WARNING, "Unable to allocate class %x\n", grObj);
         goto error;
     }
 
+    // Fetch the client object
+    status = serverGetClientUnderLock(&g_resServ, hClient, &pClient);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Unable to obtain client object\n");
+        goto error;
+    }
+
     //
     // Determine the (class + engine) handle the hardware will understand, if
     // necessary
     //
-    if (CliGetKernelChannelWithDevice(hClient,
+    if (CliGetKernelChannelWithDevice(pClient,
                                       WATCHDOG_DEVICE_ID,
                                       WATCHDOG_PUSHBUFFER_CHANNEL_ID,
                                       &pKernelChannel) != NV_OK)
@@ -1065,9 +1174,11 @@ krcWatchdogInit_IMPL
 
 error:
     NV_ASSERT(status == NV_OK);
+
     if (status != NV_OK)
     {
         pRmApi->Free(pRmApi, hClient, hClient);
+        pKernelRc->watchdog.bHandleValid = NV_FALSE;
     }
 
     portMemFree(pParams);
@@ -1307,4 +1418,11 @@ krcWatchdogWriteNotifierToGpfifo_IMPL
     SLI_LOOP_END;
 }
 
+NV_STATUS krcWatchdogGetClientHandle(KernelRc *pKernelRc, NvHandle *phClient)
+{
+    if (!pKernelRc->watchdog.bHandleValid)
+        return NV_ERR_INVALID_STATE;
 
+    *phClient = pKernelRc->watchdog.hClient;
+    return NV_OK;
+}
