@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2018-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2018-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -857,7 +857,7 @@ memCopyConstruct_IMPL
     }
 
     NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-                        memCheckCopyPermissions(pMemorySrc, pDstGpu, pDstClient->hClient), done);
+                        memCheckCopyPermissions(pMemorySrc, pDstGpu, pDstDevice), done);
 
     // Initialize Memory
     pMemoryDst->categoryClassId = pMemorySrc->categoryClassId;
@@ -891,7 +891,7 @@ memCopyConstruct_IMPL
 
         gpuSetThreadBcState(pDstGpu, NV_TRUE);
         SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY)
-        if (memdescGetPageSize64(memdescGetMemDescFromGpu(pMemoryDst->pMemDesc, pGpu), AT_GPU) == 0)
+        if (memdescGetPageSize(memdescGetMemDescFromGpu(pMemoryDst->pMemDesc, pGpu), AT_GPU) == 0)
         {
             status = memmgrSetMemDescPageSize_HAL(pGpu, GPU_GET_MEMORY_MANAGER(pGpu),
                                                   memdescGetMemDescFromGpu(pMemoryDst->pMemDesc, pGpu),
